@@ -10,14 +10,19 @@
 
 from functools import partial
 from enum import Enum, unique
+from typing import Any, Dict, Optional
+import logging
 
 from b2sdk.account_info.sqlite_account_info import SqliteAccountInfo
 from b2sdk.account_info.exception import MissingAccountData
 from b2sdk.b2http import B2Http
 from b2sdk.cache import AuthInfoCache, DummyCache
+from b2sdk.encryption.setting import EncryptionSetting
 from b2sdk.exception import (InvalidAuthToken, Unauthorized)
 from b2sdk.raw_api import ALL_CAPABILITIES, B2RawApi
 
+logger = logging.getLogger(__name__)
+
 
 @unique
 class TokenType(Enum):
@@ -108,21 +113,22 @@ class B2Session(object):
             self.cache.clear()
 
         # Authorize
-        realm_url = self.account_info.REALM_URLS[realm]
+        realm_url = self.account_info.REALM_URLS.get(realm, realm)
         response = self.raw_api.authorize_account(realm_url, application_key_id, application_key)
         allowed = response['allowed']
 
         # Store the auth data
         self.account_info.set_auth_data(
-            response['accountId'],
-            response['authorizationToken'],
-            response['apiUrl'],
-            response['downloadUrl'],
-            response['recommendedPartSize'],
-            application_key,
-            realm,
-            allowed,
-            application_key_id,
+            account_id=response['accountId'],
+            auth_token=response['authorizationToken'],
+            api_url=response['apiUrl'],
+            download_url=response['downloadUrl'],
+            minimum_part_size=response['recommendedPartSize'],
+            application_key=application_key,
+            realm=realm,
+            s3_api_url=response['s3ApiUrl'],
+            allowed=allowed,
+            application_key_id=application_key_id
         )
 
     def cancel_large_file(self, file_id):
@@ -135,7 +141,8 @@ class B2Session(object):
         bucket_type,
         bucket_info=None,
         cors_rules=None,
-        lifecycle_rules=None
+        lifecycle_rules=None,
+        default_server_side_encryption=None,
     ):
         return self._wrap_default_token(
             self.raw_api.create_bucket,
@@ -145,6 +152,7 @@ class B2Session(object):
             bucket_info=bucket_info,
             cors_rules=cors_rules,
             lifecycle_rules=lifecycle_rules,
+            default_server_side_encryption=default_server_side_encryption,
         )
 
     def create_key(
@@ -183,8 +191,11 @@ class B2Session(object):
             valid_duration_in_seconds
         )
 
-    def get_file_info(self, file_id):
-        return self._wrap_default_token(self.raw_api.get_file_info, file_id)
+    def get_file_info_by_id(self, file_id: str) -> Dict[str, Any]:
+        return self._wrap_default_token(self.raw_api.get_file_info_by_id, file_id)
+
+    def get_file_info_by_name(self, bucket_name: str, file_name: str) -> Dict[str, Any]:
+        return self._wrap_default_token(self.raw_api.get_file_info_by_name, bucket_name, file_name)
 
     def get_upload_url(self, bucket_id):
         return self._wrap_default_token(self.raw_api.get_upload_url, bucket_id)
@@ -263,9 +274,21 @@ class B2Session(object):
             prefix=prefix,
         )
 
-    def start_large_file(self, bucket_id, file_name, content_type, file_info):
+    def start_large_file(
+        self,
+        bucket_id,
+        file_name,
+        content_type,
+        file_info,
+        server_side_encryption: Optional[EncryptionSetting] = None,
+    ):
         return self._wrap_default_token(
-            self.raw_api.start_large_file, bucket_id, file_name, content_type, file_info
+            self.raw_api.start_large_file,
+            bucket_id,
+            file_name,
+            content_type,
+            file_info,
+            server_side_encryption,
         )
 
     def update_bucket(
@@ -277,6 +300,7 @@ class B2Session(object):
         cors_rules=None,
         lifecycle_rules=None,
         if_revision_is=None,
+        default_server_side_encryption: Optional[EncryptionSetting] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.update_bucket,
@@ -287,11 +311,19 @@ class B2Session(object):
             cors_rules=cors_rules,
             lifecycle_rules=lifecycle_rules,
             if_revision_is=if_revision_is,
+            default_server_side_encryption=default_server_side_encryption,
         )
 
     def upload_file(
-        self, bucket_id, file_name, content_length, content_type, content_sha1, file_infos,
-        data_stream
+        self,
+        bucket_id,
+        file_name,
+        content_length,
+        content_type,
+        content_sha1,
+        file_infos,
+        data_stream,
+        server_side_encryption: Optional[EncryptionSetting] = None,
     ):
         return self._wrap_token(
             self.raw_api.upload_file,
@@ -303,9 +335,18 @@ class B2Session(object):
             content_sha1,
             file_infos,
             data_stream,
+            server_side_encryption,
         )
 
-    def upload_part(self, file_id, part_number, content_length, sha1_sum, input_stream):
+    def upload_part(
+        self,
+        file_id,
+        part_number,
+        content_length,
+        sha1_sum,
+        input_stream,
+        server_side_encryption: Optional[EncryptionSetting] = None,
+    ):
         return self._wrap_token(
             self.raw_api.upload_part,
             TokenType.UPLOAD_PART,
@@ -314,6 +355,7 @@ class B2Session(object):
             content_length,
             sha1_sum,
             input_stream,
+            server_side_encryption,
         )
 
     def get_download_url_by_id(self, file_id):
@@ -333,6 +375,7 @@ class B2Session(object):
         content_type=None,
         file_info=None,
         destination_bucket_id=None,
+        destination_server_side_encryption: Optional[EncryptionSetting] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.copy_file,
@@ -343,6 +386,7 @@ class B2Session(object):
             content_type=content_type,
             file_info=file_info,
             destination_bucket_id=destination_bucket_id,
+            destination_server_side_encryption=destination_server_side_encryption,
         )
 
     def copy_part(
@@ -351,13 +395,15 @@ class B2Session(object):
         large_file_id,
         part_number,
         bytes_range=None,
+        destination_server_side_encryption: Optional[EncryptionSetting] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.copy_part,
             source_file_id,
             large_file_id,
             part_number,
-            bytes_range=bytes_range
+            bytes_range=bytes_range,
+            destination_server_side_encryption=destination_server_side_encryption,
         )
 
     def _wrap_default_token(self, raw_api_method, *args, **kwargs):
