@@ -41,6 +41,13 @@ from .deps import ParallelDownloader
 from .deps import SimpleDownloader
 from .deps import UploadSourceBytes
 from .deps import hex_sha1_of_bytes, TempDir
+from .deps import EncryptionAlgorithm, EncryptionSetting, EncryptionMode
+
+SSE_NONE = EncryptionSetting(mode=EncryptionMode.NONE,)
+SSE_B2_AES = EncryptionSetting(
+    mode=EncryptionMode.SSE_B2,
+    algorithm=EncryptionAlgorithm.AES256,
+)
 
 
 def write_file(path, data):
@@ -241,6 +248,36 @@ class TestListUnfinished(TestCaseWithBucket):
         return self.bucket.start_large_file(file_name, 'text/plain', {})
 
 
+class TestGetFileInfo(TestCaseWithBucket):
+    def test_version_by_name(self):
+        data = b'hello world'
+        a_id = self.bucket.upload_bytes(data, 'a').id_
+
+        info = self.bucket.get_file_info_by_name('a')
+
+        self.assertIsInstance(info, FileVersionInfo)
+        expected = (a_id, 'a', 11, None, 'b2/x-auto', 'none')
+        actual = (
+            info.id_, info.file_name, info.size, info.action, info.content_type,
+            info.server_side_encryption.mode.value
+        )
+        self.assertEqual(expected, actual)
+
+    def test_version_by_id(self):
+        data = b'hello world'
+        b_id = self.bucket.upload_bytes(data, 'b').id_
+
+        info = self.bucket.get_file_info_by_id(b_id)
+
+        self.assertIsInstance(info, FileVersionInfo)
+        expected = (b_id, 'b', 11, 'upload', 'b2/x-auto', 'none')
+        actual = (
+            info.id_, info.file_name, info.size, info.action, info.content_type,
+            info.server_side_encryption.mode.value
+        )
+        self.assertEqual(expected, actual)
+
+
 class TestLs(TestCaseWithBucket):
     def test_empty(self):
         self.assertEqual([], list(self.bucket.ls('foo')))
@@ -396,6 +433,33 @@ class TestListVersions(TestCaseWithBucket):
         except ValueError as e:
             self.assertEqual('unsupported fetch_count value', str(e))
 
+    def test_encryption(self):
+        data = b'hello world'
+        a = self.bucket.upload_bytes(data, 'a')
+        a_id = a.id_
+        self.assertEqual(a.server_side_encryption, SSE_NONE)
+        b = self.bucket.upload_bytes(data, 'b', encryption=SSE_B2_AES)
+        self.assertEqual(b.server_side_encryption, SSE_B2_AES)
+        b_id = b.id_
+        #c_id = self.bucket.upload_bytes(data, 'c', encryption=SSE_NONE).id_  # TODO
+        d_id = self.bucket.copy(a_id, 'd', destination_encryption=SSE_B2_AES).id_
+        e_id = self.bucket.copy(b_id, 'e').id_
+
+        actual = [info.server_side_encryption for info in self.bucket.list_file_versions('a')][0]
+        self.assertEqual(SSE_NONE, actual)  # bucket default
+
+        actual = [info.server_side_encryption for info in self.bucket.list_file_versions('b')][0]
+        self.assertEqual(SSE_B2_AES, actual)  # explicitly requested sse-b2
+
+        #actual = [info.server_side_encryption for info in self.bucket.list_file_versions('c')][0]
+        #self.assertEqual(SSE_NONE, actual)  # explicitly requested none
+
+        actual = [info.server_side_encryption for info in self.bucket.list_file_versions('d')][0]
+        self.assertEqual(SSE_B2_AES, actual)  # explicitly requested sse-b2
+
+        actual = [info.server_side_encryption for info in self.bucket.list_file_versions('e')][0]
+        self.assertEqual(SSE_NONE, actual)  # bucket default
+
 
 class TestCopyFile(TestCaseWithBucket):
     def test_copy_without_optional_params(self):
@@ -496,6 +560,24 @@ class TestCopyFile(TestCaseWithBucket):
         expected = [('hello_new.txt', 11, 'copy', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
+    def test_copy_encryption(self):
+        file_id = self._make_file()
+        file_info = self.bucket.copy(
+            file_id,
+            'hello_new.txt',
+            destination_encryption=SSE_B2_AES,
+        )
+        self.assertTrue(isinstance(file_info, FileVersionInfo))
+        self.assertEqual(file_info.server_side_encryption, SSE_B2_AES)
+
+        #file_info = self.bucket.copy(
+        #    file_id,
+        #    'hello_new.txt',
+        #    destination_encryption=SSE_NONE,
+        #)
+        #self.assertTrue(isinstance(file_info, FileVersionInfo))
+        #self.assertEqual(file_info.server_side_encryption, SSE_NONE)
+
     def _make_file(self, bucket=None):
         data = b'hello world'
         actual_bucket = bucket or self.bucket
@@ -508,6 +590,23 @@ class TestUpload(TestCaseWithBucket):
         file_info = self.bucket.upload_bytes(data, 'file1')
         self.assertTrue(isinstance(file_info, FileVersionInfo))
         self._check_file_contents('file1', data)
+        self.assertEqual(file_info.server_side_encryption, SSE_NONE)
+
+    def test_upload_bytes_sse_b2(self):
+        data = b'hello world'
+        file_info = self.bucket.upload_bytes(data, 'file1', encryption=SSE_B2_AES)
+        self.assertTrue(isinstance(file_info, FileVersionInfo))
+        self.assertEqual(file_info.server_side_encryption, SSE_B2_AES)
+
+    def test_upload_local_file_sse_b2(self):
+        with TempDir() as d:
+            path = os.path.join(d, 'file1')
+            data = b'hello world'
+            write_file(path, data)
+            file_info = self.bucket.upload_local_file(path, 'file1', encryption=SSE_B2_AES)
+            self.assertTrue(isinstance(file_info, FileVersionInfo))
+            self.assertEqual(file_info.server_side_encryption, SSE_B2_AES)
+            self._check_file_contents('file1', data)
 
     def test_upload_bytes_progress(self):
         data = b'hello world'
@@ -520,8 +619,12 @@ class TestUpload(TestCaseWithBucket):
             path = os.path.join(d, 'file1')
             data = b'hello world'
             write_file(path, data)
-            self.bucket.upload_local_file(path, 'file1')
+            file_info = self.bucket.upload_local_file(path, 'file1')
             self._check_file_contents('file1', data)
+            self.assertTrue(isinstance(file_info, FileVersionInfo))
+            self.assertEqual(file_info.server_side_encryption, SSE_NONE)
+            print(file_info.as_dict())
+            self.assertEqual(file_info.as_dict()['serverSideEncryption'], {'mode': 'none'})
 
     @pytest.mark.skipif(platform.system() == 'Windows', reason='no os.mkfifo() on Windows')
     def test_upload_fifo(self):
@@ -548,7 +651,7 @@ class TestUpload(TestCaseWithBucket):
                 self.bucket.upload_local_file(
                     path,
                     'file123',
-                    sha1_sum='abcabcabc',
+                    sha1_sum='abcd' * 10,
                 )
 
     def test_upload_one_retryable_error(self):
